Bagel codec README
===

Bagel is a codec aimed at compressing the gfx data of FC/NES games. It's designed to decompress at high speeds[^1] while also allowing partial decoding for the limited bandwidth of an active NES PPU. The compressed size averages between [pb53] and [tokumaru's compression].

[^1]: only 3x as slow as a memory copy!

[pb53]: http://wiki.nesdev.com/w/index.php/PB53
[tokumaru's compression]: http://forums.nesdev.com/viewtopic.php?f=2&t=5860

## Format overview

Bagel operates by groups of 8 bytes called planes, and groups of 8 planes (or 64 bytes) called blocks. Each plane has a 2 or 10 bit header with the first 2 being packed into the block header. Finally the sequence of blocks are prepended with a single byte block count.

### Block count
the number compressed blocks that follows. '0' is a special mode that interleaves two pages of 64 blocks.

### Block header
The 2 mode bits of each of the 8 planes, packed in MSB bit and byte order. If the planes modes repeats in groups of 2, then a compact 1 byte block header is used.

Plane mode `00`
: The entire plane is `0x00` bytes.

Plane mode `01`
: The plane is just 8 new bytes from the data stream.

Plane mode `10`
: Include a 8 bit plane header that references the previous byte for each zero bit. The first referenced byte implied as `0x00`.

Plane mode `11`
: Include a 8 bit plane header that references the previous *plane* for each zero bit. If the first plane of the block has this mode then the whole block is duplicated from the previous block.

### Plane header
For each bit in the plane header: `0` means duplicate the referenced byte depending on the plane mode; `1` means take a new byte from the data stream.

if the whole header is `0xff`: for plane mode `10`, this means the entire plane is `0xff` bytes; for plane mode `11`, this means the entire plane is inverted from the previous plane.

If the whole header is `0x00` and the mode is `10` then plane is duplicated from the previous block.


## List of files

README.txt
: This file.

bagel.py
: A compressor program written in Python 3.

bagel.s
: The main block decompressor with some helper subroutines
  written in 6502 assembly (ca65 dialect).

ntd.py
: A Python 3 compressor for supplemental nametable compression.

ntd.s
: The 6502 decompressor of the above nametable compression.

pkb.s
: Small PackBits decompressor that assembles to only 46 bytes!

demo/
: An example NES project demonstrating various setups of the
  block decompressor. 

### Demo credits.
  The bottom of the test tileset includes [Generitiles] by Drag, and the characters from [Chase] by Shiru. The large numbers in the game is the font [Fizzter] by tepples. The game idea is from "Bagel" as described by a book titled "What to Do After You Hit Return".

[Generitiles]: http://wiki.nesdev.com/w/index.php/Placeholder_graphics
[Chase]: https://shiru.untergrund.net/articles/programming_nes_games_in_c.htm
[Fizzter]: http://forums.nesdev.com/viewtopic.php?f=21&t=12051


## Versions
- 2016-09-19: Initial release


---
By Johnathan Roatch, 2016

This text is licensed to the public under the
[Creative Commons Attribution 4.0 License][CC BY 4.0].

[CC BY 4.0]: http://creativecommons.org/licenses/by/4.0/
